package ds;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

/** Base class for all forms used in the application.
  * Collects some common code.
  * @author Daniel Stoinski
  * @version $Revision: 26 $
  */
public abstract class BaseForm extends Form implements CommandListener {

  /** The parent midlet.
    */
  protected DictMidlet m_parent;

  /** Command actions.
    * The method in this class recognizes only Command.EXIT, which causes
    * displaying of the main form.
    * @param c  the command.
    * @param s  the display of this form, usually ignored.
    */
  public void commandAction(Command c, Displayable s) {
    Config   cfg;
    MainForm mfrm;

    if (c.getCommandType() == Command.EXIT) {
      if (this.m_parent != null) {
        cfg = this.m_parent.getCfg();
        cfg.save();
        mfrm = this.m_parent.getMainForm();
        if (mfrm != null) 
         Display.getDisplay(this.m_parent).setCurrent(mfrm);
      }
    }
  } // commandAction()

  /** Initializes the form.
    * Initializes the parent, sets title and adds command EXIT.
    * @param aTitle  the title of the form, passed to the base constructor.
    * @param aParent the parent midlet.
    */
  public BaseForm(String aTitle, DictMidlet aParent) {
    super(aTitle);
    this.m_parent = aParent;
    this.addCommand(new Command("Exit", Command.EXIT, 0));
    this.setCommandListener(this);
  } // BaseForm()

  /** Prints the given message in an alert box.
    * Presently to stdout. Logging into form item planed.
    * @param msg  the message to log.
    */
  public synchronized void message(Object msg) {
    Alert alert;

    if (msg != null) {
      alert = new Alert("Warning", msg.toString(), null, AlertType.INFO);
      alert.setTimeout(Alert.FOREVER);
      if (this.m_parent != null) {
        Display.getDisplay(this.m_parent).setCurrent(alert, this);
      }
    }
  } // message()

  /** Shows the given form.
    * @param frm the form to show
    */
  protected void showForm(Form frm) {
    if (this.m_parent != null && frm != null) {
      Display.getDisplay(this.m_parent).setCurrent(frm);
    }
  } // showForm()

} // class BaseForm

