package ds;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import javax.microedition.rms.RecordStore;

/** Configuration of the application.
  * Returns values read from rms or plausible defaults.
  * @author Daniel Stoinski
  * @version $Revision: 26 $
  */
public class Config {

  /** Default value for the dictionary file to import.
    */
  public static final String DEF_DICT_FILE = "file:///karta/infile.txt";

  /** Default value for the index file created with PrepDict.
    */
  public static final String DEF_IDX_FILE = "file:///karta/index.txt";

  /** Default value of the database with the imported dictionary.
    */
  public static final String DEF_DB_NAME = "file:///karta/test.db";

  /** Index of the parameter "dictionary file" in the config parameters.
    */
  public static final int DICTFILE = 0;

  /** Index of the parameter "index file" in the config parameters.
    */
  public static final int IDXFILE = 1;

  /** Index of the parameter "database name" in the config parameters.
    */
  public static final int DBNAME = 2;

  /** Number of configuration parameters.
    */
  private static final int NUMPARAMS = 3;

  /** Record store name.
    */
  private static final String RSNAME = "dictcfg";

  /** Array of config parameters.
    */
  private String[] m_parameters;

  /** If the content of the configuration has changed since last write.
    */
  private boolean m_changed;

  /** Initializes the configuration.
    */
  public Config() {
    this.m_changed = false;
    this.m_parameters = new String[Config.NUMPARAMS];
    this.m_parameters[Config.DICTFILE] = Config.DEF_DICT_FILE;
    this.m_parameters[Config.IDXFILE] = Config.DEF_IDX_FILE;
    this.m_parameters[Config.DBNAME] = Config.DEF_DB_NAME;
    try { this.load(); } catch(Exception excp) {  }
  } // Config()

  /** Sets the parameter having the given index to the specified value.
    * The method does nothing for wrong configuration parameters.
    * @param paramIdx  the index of the configuration parameter.
    * @param value     the value to set.
    */
  public void set(int paramIdx, String value) {
    try {
      if (!this.m_parameters[paramIdx].equals(value)) {
        this.m_parameters[paramIdx] = value;
        this.m_changed = true;
      }
    }
    catch(ArrayIndexOutOfBoundsException excp) {
    }
  } // set()

  /** Returns the parameter having the given index.
    * @param paramIdx  the index of the configuration parameter.
    * @return the value of the requested parameter or an empty string for
    *         wrong configuration parameters.
    */
  public String get(int paramIdx) {
    String retval;

    try {
      retval = this.m_parameters[paramIdx];
    }
    catch(ArrayIndexOutOfBoundsException excp) {
      retval = "";
    }
    return retval;
  } // get()

  /** Loads the configuration from the record set 0.
    */
  private void load() {
    ByteArrayInputStream bis;
    DataInputStream      dis;
    RecordStore          rs;
    byte[]               b;
    int                  i;

    rs = null;
    try {
      rs = RecordStore.openRecordStore(Config.RSNAME, false);
      if (rs != null && rs.getNumRecords() > 0) {
        b = rs.getRecord(1);
        bis = new ByteArrayInputStream(b);
        dis = new DataInputStream(bis);
        for(i = 0; i < this.m_parameters.length; i++)
          this.m_parameters[i] = dis.readUTF();
        dis.close();
      }
    }
    catch(Exception e) { }
    if (rs != null) {
      try { rs.closeRecordStore(); } catch(Exception excp) { }
    }
  } // load()

  /** Saves the configuration to rms if it has changed since last write.
    */
  public void save() {
    ByteArrayOutputStream bos;
    DataOutputStream      dos;
    RecordStore           rs;
    byte[]                b;
    int                   i;

    if (this.m_changed) {
      rs = null;
      try {
        rs = RecordStore.openRecordStore(Config.RSNAME, true);
        if (rs != null) {
          bos = new ByteArrayOutputStream();
          dos = new DataOutputStream(bos);
          for(i = 0; i < this.m_parameters.length; i++)
            dos.writeUTF(this.m_parameters[i]);
          b = bos.toByteArray();
          dos.close();
          if (rs.getNumRecords() > 0) rs.setRecord(1, b, 0, b.length);
          else {
            i = rs.addRecord(b, 0, b.length);
          }
          this.m_changed = false;
        }
      }
      catch(Exception e) { }
      if (rs != null) {
        try { rs.closeRecordStore(); } catch(Exception excp) { }
      }
    } // if (this.m_changed)
  } // save()
} // class Config

