package ds;
import org.garret.perst.IInputStream;
import org.garret.perst.IOutputStream;
import org.garret.perst.Persistent;
import org.garret.perst.Storage;

/** The dictionary entry consists of the line number in the dictionary line
  * and the whole dictionary line.
  * The dictionary lines are assumed to be whole lines from the original
  * dict.cc file.
  * @author Daniel Stoinski
  * @version $Revision: 30 $
  */
public class DictEntry extends Persistent {

  /** The line number of the dictionary line.
    */
  private int m_lineno;

  /** The whole dictionary line.
    */
  private String m_line;

  /** Initializes the object with empty invalid values.
    */
  public DictEntry() {
    super();
    this.m_lineno = -1;
    this.m_line = "";
  }

  /** Initializes the index and line with the given values.
    * @param aLineNo  the line number of the dictionary line
    * @param aLine    the line.
    */
  public DictEntry(int aLineNo, String aLine) {
    super();
    this.m_lineno = aLineNo;
    this.m_line = aLine == null? "" : aLine;
  } // DictEntry()

  /** Initializes for the given storage, line number and dictionary line.
    * @param aStorage  the Prst storage.
    * @param aLineNo   the line number of the dictionary line.
    * @param aLine     the line.
    */
  public DictEntry(Storage aStorage, int aLineNo, String aLine) {
    super(aStorage);
    this.m_lineno = aLineNo;
    this.m_line = aLine == null? "" : aLine;
  } // DictEntry()

  /** The index.
    * @return the index for this dictionary entry.
    */
  public int getLineNo() {
    return this.m_lineno;
  } // getIndex()

  /** The line.
    * @return the whole dictionary line.
    */
  public String getLine() {
    return this.m_line;
  } // getLine()

  /** The string representation of the object.
    * @return string consisting of the index and line separated by one
    *         whitespace.
    */
  public String toString() {
    StringBuffer sb;

    sb = new StringBuffer();
    sb.append(this.m_lineno).append(' ').append(this.m_line);
    return sb.toString();
  } // toString()

  /** Deserializes the object.
    * Reads the object from the given input stream. First it reads the index
    * (line number) and then the dictionary line.
    * @param in   the input stream to read the object from.
    */
  public void readObject(IInputStream in) {
    this.m_lineno = in.readInt();
    this.m_line = in.readString();
  } // readObject()

  /** Serializes the object.
    * Writes the object to the given stream, first index (line number)
    * then the dictionary line.
    * @param out the stream to write the object to.
    */
  public void writeObject(IOutputStream out) {
    out.writeInt(this.m_lineno);
    out.writeString(this.m_line);
  } // writeObject()
} // class DictEntry

