package ds;
import org.garret.perst.IInputStream;
import org.garret.perst.IOutputStream;
import org.garret.perst.Index;
import org.garret.perst.Persistent;
import org.garret.perst.Storage;
import org.garret.perst.Types;

/** The main object for accessing Perst entries.
  * @author Daniel Stoinski
  * @version $Revision: 31 $
  */
public class DictRoot extends Persistent {

  /** The keyword index of dict index entries.
    */
  private Index m_kwidx;

  /** The line number index of dictionary entries.
    */
  private Index m_dictidx;

  /** Initializes the object with null indexes.
    */
  public DictRoot() {
    super();
    this.m_kwidx = null;
    this.m_dictidx = null;
  } // DictRoot()

  /** Constructor called while database initialization.
    * Creates unique indexes for the keywords and line numbers.
    * @param aStorage  the Pers storage object.
    */
  public DictRoot(Storage aStorage) {
    super(aStorage);
    this.m_kwidx = aStorage.createIndex(Types.String, true);
    this.m_dictidx = aStorage.createIndex(Types.Int, true);
  } // DictRoot()

  /** Deserializes the object.
    * Retrieves the indexes from the input stream.
    * @param in  the input stream to read from.
    */
  public void readObject(IInputStream in) {
    this.m_kwidx = (Index) in.readObject();
    this.m_dictidx = (Index) in.readObject();
  } // readObject()

  /** Serializes the object.
    * Writes the indexes to the given stream.
    * @param out the output stream to write to.
    */
  public void writeObject(IOutputStream out) {
    out.writeObject(this.m_kwidx);
    out.writeObject(this.m_dictidx);
  } // writeObject()

  /** Returns the keyword index.
    * @return the index.
    */
  public Index getKwIdx() {
    return this.m_kwidx;
  } // getKwIdx()

  /** Returns the dictionary entry index.
    */
  public Index getDictIdx() {
    return this.m_dictidx;
  } // getDictIdx()
} // class DictRoot

