package ds;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.ImageItem;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemCommandListener;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileSystemRegistry;
import javax.microedition.io.file.FileConnection;
import java.util.Enumeration;

/** File system browser.
  * @author Daniel Stoinski
  * @version $Revision: 36 $
  */
public class FileForm extends BaseForm implements ItemCommandListener {

  /** Layout for file and directory items in the form.
    */
  private static final int ITEM_LAYOUT = Item.LAYOUT_LEFT |
                                         Item.LAYOUT_NEWLINE_AFTER |
                                         Item.LAYOUT_EXPAND;

  /** Parent form.
    * Return to this form if the user pressed exit.
    */
  private BaseForm m_parentform;

  /** Parent directory of the directory currently set in setPath.
    * @see #parent
    */
  private String m_parentdir;

  /** Image used for showing directories.
    */
  private Image m_imgdir;

  /** Image used for the current directory.
    */
  private Image m_imgcurdir;

  /** Image used for a file.
    */
  private Image m_imgfile;

  /** The text field, where to store the selected file or directory name.
    * Passed to the constructor. No meaning, if null.
    */
  private TextField m_resfld;

  /** Current selection.
    */
  private StringItem m_cursel;

  /** Initializes the form.
    * No specific directory or file yet.
    * @param aParent     the parent midlet
    * @param aParentForm the parent form, return to this form on exit.
    * @param aResFld     the text field where to store the selected file
    *                    or directory, if pressed OK.
    */
  public FileForm(DictMidlet aParent, BaseForm aParentForm, TextField aResFld) {
    super("select file", aParent);

    this.m_parentform = aParentForm;
    this.m_parentdir = null;
    this.m_imgdir = null;
    this.m_imgcurdir = null;
    this.m_imgfile = null;
    this.m_resfld = aResFld;
    this.m_cursel = null;
    this.addCommand(new Command("Back", Command.SCREEN, 0));
    this.addCommand(new Command("OK", Command.OK, 0));
    try {
      this.m_imgdir = Image.createImage("/dir.png");
      this.m_imgcurdir = Image.createImage("/curdir.png");
      this.m_imgfile = Image.createImage("/file.png");
    }
    catch(Exception excp) {}
  } // FileForm

  /** Command action.
    * EXIT - return to the parent form, OK - select the directory or file.
    * SCREEN - select the parent directory.
    * @param c  used in order to find the command type.
    * @param s  ignored.
    */
  public void commandAction(Command c, Displayable s) {
    switch(c.getCommandType()) {
      case Command.EXIT :
        if (this.m_parentform != null) this.showForm(this.m_parentform);
        else super.commandAction(c, s);
        break;
      case Command.OK :
        if (this.m_cursel != null) {
          if (this.m_resfld != null)
            this.m_resfld.setString(this.m_cursel.getText());
          if (this.m_parentform != null) this.showForm(this.m_parentform);
          else super.commandAction(c, s);
        }
        break;
      case Command.SCREEN :
        this.setPath(this.m_parentdir);
        break;
      default :
        super.commandAction(c, s);
        break;
    } // switch
  } // commandAction()

  /** Opens a file connection to the given file.
    * Returns null on any kind of errors, if the file name is wrong
    * or doesn't exist. Automatically prepends the given file name
    * with file:/// if not already done, automatically tries any 
    * permutation with and without the trailing slashes in order to
    * find out, if the name is for a directory or a file.
    * @param path   the name of the file to check
    * @return the file connection for the given file or null, if the
    *         file doesn't exist.
    */
  private static FileConnection open(String path) {
    FileConnection retval, fc;
    StringBuffer   sb;
    String         p;
    int            l;

    retval = null;
    if (path != null && path.length() > 0) {
      p = path.trim();
      sb = new StringBuffer(p);
      if (!p.startsWith("file://")) {
        while(sb.charAt(0) == '/') sb.deleteCharAt(0);
        sb.insert(0, "file:///");
        p = sb.toString();
      }
      try { fc = (FileConnection) Connector.open(p); }
      catch(Exception excp) { fc = null; };
      if (fc != null && fc.exists()) retval = fc;
      else {
        l = sb.length() - 1;
        if (sb.charAt(l) == '/') {
          do {
            sb.deleteCharAt(l);
            l--;
          } while (sb.charAt(l) == '/');
        }
        else
          sb.append('/');
        p = sb.toString();
        try { fc = (FileConnection) Connector.open(p); }
        catch(Exception excp) { fc = null; };
        if (fc != null && fc.exists()) retval = fc;
      }
    }
    return retval;
  } // open()

  /** Returns the parent directory to the given file.
    * @param fc the file or directory for which to look for
    *           a parent.
    * @return the parent directory URL or null, if the parent can't be found.
    */
  private static String parent(FileConnection fc) {
    String retval, url;
    int    l, i;

    retval = null;
    if (fc != null) {
      url = fc.getURL().trim();
      l = url.length();
      if (l > 0) {
        i = url.lastIndexOf('/');
        if (i == l-1) {
          url = url.substring(0, i);
          i = url.lastIndexOf('/');
        }
        if (i > 0) {
          url = url.substring(0, i+1);
          if (!(url.equals("file://") ||
                url.equals("file:/") ||
                url.equals("file:"))) retval = url;
        }
      }
    }
    return retval;
  } // parent()

  /** Creates an image item for a file or directory and appends it to the form.
    * If the given path ends with a slash, then a directory item is
    * created, else a file item. The items will get proper commands
    * and listeners set.
    * @param aBaseDir  The directory to prepend for the full file or
    *                  directory name. It must have a trailing slasg.
    * @param aName     The file or directory name. Directories must
    *                  have a trailing slash, files must not.
    *                  This must be only a name relative to aParentDir.
    */
  private void addItem(String aBaseDir, String aName) {
    StringBuffer   sb;
    ImageItem      item;

    if (aName.endsWith("/"))
      item = new ImageItem(aName, this.m_imgdir, FileForm.ITEM_LAYOUT, "dir");
    else
      item = new ImageItem(aName, this.m_imgfile, FileForm.ITEM_LAYOUT, "file");
    sb = new StringBuffer(aBaseDir); sb.append(aName);
    item.setDefaultCommand(new Command(sb.toString(), Command.ITEM, 0));
    item.setItemCommandListener(this);
    this.append(item);
  } // crtItem()

  /** Sets the file or directory to show.
    * @param aPath the path to set. May begin with file://.
    */
  public void setPath(String aPath) {
    Enumeration    e;
    FileConnection fc;
    ImageItem      item;
    String         dir;

    this.deleteAll();
    fc = FileForm.open(aPath);
    this.m_parentdir = FileForm.parent(fc);
    if (fc == null) {
      dir = "file:///";
      this.m_cursel = null;
      e = FileSystemRegistry.listRoots();
      while(e.hasMoreElements())
        this.addItem(dir, e.nextElement().toString());
    }
    else {
      dir = fc.getURL();
      this.m_cursel = new StringItem("Current selection", dir);
      this.append(m_cursel);
      if (fc.isDirectory()) {
        try {
          e = fc.list("*", true);
          while(e.hasMoreElements())
            this.addItem(dir, e.nextElement().toString());
        }
        catch(Exception excp) {
          this.message(excp.toString());
        }
      } // if (fc.isDirectory())
    } // ekse
  } // setPath()

  /** Selects the content of the given item as the current directory or file
    * name.
    * @param c  checks, if the command is Command.ITEM. The method does nothing,
    *           if it isn't.
    * @param item sets the current directory or file name to the content of
    *             this item.
    */
  public void commandAction(Command c, Item item) {
    if (c.getCommandType() == Command.ITEM) {
      this.setPath(c.getLabel());
    }
  } // commandAction()

} // FileForm

