package ds;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Gauge;
import javax.microedition.lcdui.TextField;

/** Load form used for loading dictionary into the DB.
  * @author Daniel Stoinski
  * @version $Revision: 36 $
  */
public class LoadForm extends BaseForm implements IDictForm {

  /** Maximum text length for text boxes.
    */
  private static final int MAX_TXTLEN = 256;

  /** Text box for the name of the dictionary input file.
    */
  private TextField m_tbinpfile;

  /** Text box for the name of the index file.
    * The index file has to be created using PrepDict on the dektop
    * PC prior to import it on the mobile device.
    */
  private TextField m_tbidxfile;

  /** Text box for the name of the DB to which to import the dictionary.
    */
  private TextField m_tbdbname;

  /** Progress of the import process.
    */
  private Gauge m_progress;

  /** Initializes the form.
    * @param aParent  the parent midlet, if not null, closing this
    *                 form causes displaying the parent.
    */
  public LoadForm(DictMidlet aParent) {
    super("Load dictionary", aParent);

    String inpfile, idxfile, dbname;
    Config cfg;

    if (this.m_parent != null) {
      cfg = this.m_parent.getCfg();
      inpfile = cfg.get(Config.DICTFILE);
      idxfile = cfg.get(Config.IDXFILE);
      dbname = cfg.get(Config.DBNAME);
    }
    else {
      inpfile = "";
      idxfile = "";
      dbname = "";
    }
    this.addCommand(new Command("Load", Command.SCREEN, 0));
    this.addCommand(new Command("Select input", Command.SCREEN, 1));
    this.addCommand(new Command("Select index", Command.SCREEN, 2));
    this.addCommand(new Command("Select DB", Command.SCREEN, 3));
    this.m_tbinpfile = new TextField("Input file", 
                                     inpfile,
                                     LoadForm.MAX_TXTLEN,
                                     TextField.URL);
    this.m_tbidxfile = new TextField("Index file",
                                     idxfile,
                                     LoadForm.MAX_TXTLEN,
                                     TextField.URL);
    this.m_tbdbname = new TextField("DB",
                                    dbname,
                                    LoadForm.MAX_TXTLEN,
                                    TextField.ANY);
    this.m_progress = new Gauge("Progress", false, 100, 0);
    this.append(this.m_tbinpfile);
    this.append(this.m_tbidxfile);
    this.append(this.m_tbdbname);
    this.append(this.m_progress);
  } // LoadForm()

  /** Command actions.
    * Returns to the main form of the midlet for Command.Exit (performed in the
    * base class).
    * Activates dictionary importing for Command.SCREEN.
    * @param c used in order to find the command type.
    * @param s ignored.
    * @see Import
    */
  public void commandAction(Command c, Displayable s) {
    FileForm   ff;
    Import     idct;
    Config     cfg;
    String     inpfile, idxfile, dbname;

    switch(c.getCommandType()) {
      case Command.SCREEN :
        switch(c.getPriority()) {
          case 0 : // load - import database
            try {
              inpfile = this.m_tbinpfile.getString();
              idxfile = this.m_tbidxfile.getString();
              dbname = this.m_tbdbname.getString();
              if (this.m_parent != null) {
                cfg = this.m_parent.getCfg();
                cfg.set(Config.DICTFILE, inpfile);
                cfg.set(Config.IDXFILE, idxfile);
                cfg.set(Config.DBNAME, dbname);
              }
              this.m_progress.setValue(0);
              idct = new Import(this, inpfile, idxfile, dbname);
              idct.start();
            }
            catch(Exception excp) {
              this.message(excp);
            }
            break;
          case 1 : // select input file
            ff = new FileForm(this.m_parent, this, this.m_tbinpfile);
            ff.setPath(this.m_tbinpfile.getString());
            this.showForm(ff);
            break;
          case 2 : // select index file
            ff = new FileForm(this.m_parent, this, this.m_tbidxfile);
            ff.setPath(this.m_tbidxfile.getString());
            this.showForm(ff);
            break;
          default : // select DB file
            ff = new FileForm(this.m_parent, this, this.m_tbdbname);
            ff.setPath(this.m_tbdbname.getString());
            this.showForm(ff);
            break;
        } // switch
        break;
      case Command.EXIT :
        inpfile = this.m_tbinpfile.getString();
        idxfile = this.m_tbidxfile.getString();
        dbname = this.m_tbdbname.getString();
        if (this.m_parent != null) {
          cfg = this.m_parent.getCfg();
          cfg.set(Config.DICTFILE, inpfile);
          cfg.set(Config.IDXFILE, idxfile);
          cfg.set(Config.DBNAME, dbname);
        }
        super.commandAction(c, s);
        break;
      default :
        super.commandAction(c, s);
        break;
    } // switch
  } // commandAction()

  /** Shows the progress in the gauge.
    * @param percents  the progress in percents. Only values from 0 to incl.
    *                  100 used, else ignored.
    */
  public synchronized void progress(int percents) {
    int max, val;

    if (percents >= 0 && percents <= 100) {
      max = this.m_progress.getMaxValue();
      val = (int) (max * percents * 0.01);
      if (val != this.m_progress.getValue()) this.m_progress.setValue(val);
    }
  } // progress()
} // class LoadForm

