package ds;
import org.garret.perst.Index;
import org.garret.perst.Iterator;
import org.garret.perst.Key;
import org.garret.perst.Storage;
import org.garret.perst.StorageFactory;

/** Searches the database for keywords.
  * @author Daniel Stoinski
  * @version $Revision: 31 $
  */
public class Search implements Runnable {

  /** Present search thread.
    */
  private Thread m_thr;

  /** m_goon false means, that searching is interrupted.
    */
  private boolean m_goon;

  /** The dictionary database.
    */
  private Storage m_db;

  /** The results consumer.
    */
  private IDictResults m_reshandle;

  /** The keyword to search.
    */
  private String m_keyword;

  /** Initializes the object for the given results consumer and the database.
    * @param aResHandle results consumer for retrieving search results.
    * @param aDBName    the dictionary database name.
    */
  public Search(IDictResults aResHandle, String aDBName) {
    this.m_goon = false;
    this.m_thr = null;
    this.m_reshandle = aResHandle;
    this.m_db = StorageFactory.getInstance().createStorage();
    this.m_db.open(aDBName, Import.PAGE_POOL_SIZE);
  } // Search()

  /** Closes the database.
    * Call this method, if no more searching will be performed.
    */
  public void close() {
    if (this.m_db != null) this.m_db.close();
    this.stop();
    this.m_db = null;
    this.m_reshandle = null;
  } // close()

  /** Stops any running searches.
    */
  public void stop() {
    this.m_goon = false;
    this.m_keyword = null;
    if (this.m_thr != null) try { this.m_thr.join(); } catch(Exception excp) {}
    this.m_thr = null;
  } // stop()

  /** Searches for the keyword in m_keyword.
    * Calls m_reshandle.addResult() for each result found.
    */
  private synchronized void dosearch() {
    DictIndexEntry die;
    DictEntry      de;
    DictRoot       root;
    Index          kwidx, dictidx;
    Iterator       it;
    int[]          lines;
    int            i;

    if (this.m_goon && this.m_db != null && this.m_reshandle != null &&
        this.m_keyword != null && this.m_keyword.length() > 0) {
      root = (DictRoot) this.m_db.getRoot();
      if (root != null) {
        kwidx = root.getKwIdx();
        dictidx = root.getDictIdx();
        it = kwidx.prefixIterator(this.m_keyword.toUpperCase().trim());
        if (it != null) {
          while(this.m_goon && it.hasNext()) {
            die = (DictIndexEntry) it.next();
            lines = die.getLines();
            if (lines != null) {
              for(i = 0; i < lines.length && this.m_goon; i++) {
                de = (DictEntry) dictidx.get(new Key(lines[i]));
                if (de != null) {
                  this.m_reshandle.addResult(de.getLine());
                }
              } // for i
            }
          } // while
        } // if
      } // if
      this.m_goon = false;
    } // if
  } // search()

  /** Called if running as thread calls dosearch.
    */
  public void run() {
    try {
      this.m_goon = true;
      this.dosearch();
    }
    catch(Exception excp) {
      if (this.m_reshandle != null) this.m_reshandle.addResult(excp.toString());
    }
  } // run()

  /** Searches for the given keyword.
    * @param aKeyWord the keyword to search
    */
  public void search(String aKeyWord) {
    this.stop();
    this.m_thr = new Thread(this);
    this.m_keyword = aKeyWord;
    this.m_thr.start();
  }
} // class Search

