package ds;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.TextField;

/** For for searching keywords.
  * This is the main form for finding the translation based on the
  * imported dictionary.
  * @author Daniel Stoinski
  * @version $Revision: 36 $
  */
public class SearchForm extends BaseForm implements IDictResults {

  /** Maximum length of the word to search.
    */
  private static final int MAXINPLEN = 64;

  /** Maximum number of answers shown in the form.
    */
  private static final int MAXANSWERS = 100;

  /** The search engine of the dictionary.
    */
  private Search m_search;

  /** The input text field for entering words to search.
    */
  private TextField m_input;

  /** Initializes the search form.
    * @param aParent  the parent midlet.
    */
  public SearchForm(DictMidlet aParent) {
    super("Search", aParent);

    String dbname;

    this.m_input = new TextField("Word",
                                 "",
                                 SearchForm.MAXINPLEN,
                                 TextField.ANY);
    this.addCommand(new Command("Search", Command.SCREEN, 0));
    this.emptyForm();
    this.m_search = null;
    if (this.m_parent != null) {
      try {
        dbname = m_parent.getCfg().get(Config.DBNAME);
        this.m_search = new Search(this, dbname);
      }
      catch(Exception excp) {
        this.message(excp.toString());
      }
    }
  } // SearchForm()

  /** Creates an empty form containing only the input text field.
    */
  private void emptyForm() {
    this.deleteAll();
    this.append(this.m_input);
  } // emptyForm()

  public void addResult(String aResult) {
    if (aResult != null && aResult.length() > 0) this.append(aResult);
  } // addResult()

  /** Command actions.
    * Command.EXIT - return to the main form, performed in base.commandAction().
    * Command.SCREEN - search (translate).
    * @param c the command.
    * @param s ignored.
    */
  public void commandAction(Command c, Displayable s) {
    switch(c.getCommandType()) {
      case Command.SCREEN :
        if (this.m_search != null) {
          this.emptyForm();
          this.m_search.search(this.m_input.getString());
        }
        break;
      case Command.EXIT :
        if (this.m_search != null) {
          this.m_search.close();
          this.m_search = null;
        }
        super.commandAction(c, s);
        break;
      default :
        super.commandAction(c, s);
        break;
    } // switch
  } // commandAction()

} // class SearchForm

